@@ -1057,7 +1057,6 @@ pub enum ExternalMsg {
     ResetInputBuffer,
 
     /// Switch input mode.
-    /// This will reset the input buffer and call `Refresh` automatically.
     ///
     /// > **NOTE:** To be specific about which mode to switch to, use `SwitchModeBuiltin` or
     /// `SwitchModeCustom` instead.
@@ -1066,17 +1065,18 @@ pub enum ExternalMsg {
     SwitchMode(String),
 
     /// Switch to a builtin mode.
-    /// This will reset the input buffer and call `Refresh` automatically.
     ///
     /// **Example:** `SwitchModeBuiltin: default`
     SwitchModeBuiltin(String),
 
     /// Switch to a custom mode.
-    /// This will reset the input buffer and call `Refresh` automatically.
     ///
     /// **Example:** `SwitchModeCustom: my_custom_mode`
     SwitchModeCustom(String),
 
+    /// Pop the last mode from the history and switch to it.
+    PopMode,
+
     /// Switch layout.
     /// This will call `Refresh` automatically.
     ///
@@ -1237,6 +1237,11 @@ pub enum ExternalMsg {
     /// **Example:** `LogSuccess: satellite reached destination`.
     LogSuccess(String),
 
+    /// Log an warning message.
+    ///
+    /// **Example:** `LogWarning: satellite is heating`
+    LogWarning(String),
+
     /// Log an error message.
     ///
     /// **Example:** `LogError: satellite crashed`
@@ -1327,6 +1332,7 @@ impl Task {
 #[serde(rename_all = "UPPERCASE")]
 pub enum LogLevel {
     Info,
+    Warning,
     Success,
     Error,
 }
@@ -1367,6 +1373,7 @@ impl std::fmt::Display for Log {
     fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
         let level_str = match self.level {
             LogLevel::Info => "INFO   ",
+            LogLevel::Warning => "WARNING",
             LogLevel::Success => "SUCCESS",
             LogLevel::Error => "ERROR  ",
         };
@@ -1425,7 +1432,9 @@ pub struct App {
     pipe: Pipe,
     explorer_config: ExplorerConfig,
     logs: Vec<Log>,
+    logs_hidden: bool,
     history: History,
+    last_modes: Vec<Mode>,
 }
 
 impl App {
@@ -1525,7 +1534,9 @@ impl App {
             pipe: Pipe::from_session_path(&session_path)?,
             explorer_config,
             logs: Default::default(),
+            logs_hidden: Default::default(),
             history: Default::default(),
+            last_modes: Default::default(),
         }
         .change_directory(&pwd)?;
 
@@ -1622,6 +1633,7 @@ impl App {
                 ExternalMsg::SwitchMode(mode) => self.switch_mode(&mode),
                 ExternalMsg::SwitchModeBuiltin(mode) => self.switch_mode_builtin(&mode),
                 ExternalMsg::SwitchModeCustom(mode) => self.switch_mode_custom(&mode),
+                ExternalMsg::PopMode => self.pop_mode(),
                 ExternalMsg::SwitchLayout(mode) => self.switch_layout(&mode),
                 ExternalMsg::SwitchLayoutBuiltin(mode) => self.switch_layout_builtin(&mode),
                 ExternalMsg::SwitchLayoutCustom(mode) => self.switch_layout_custom(&mode),
@@ -1657,6 +1669,7 @@ impl App {
                 ExternalMsg::ClearNodeSorters => self.clear_node_sorters(),
                 ExternalMsg::LogInfo(l) => self.log_info(l),
                 ExternalMsg::LogSuccess(l) => self.log_success(l),
+                ExternalMsg::LogWarning(l) => self.log_warning(l),
                 ExternalMsg::LogError(l) => self.log_error(l),
                 ExternalMsg::Quit => self.quit(),
                 ExternalMsg::PrintResultAndQuit => self.print_result_and_quit(),
@@ -1691,7 +1704,16 @@ impl App {
                     None
                 }
             })
-            .unwrap_or_else(|| default.map(|a| a.messages().clone()).unwrap_or_default());
+            .or_else(|| default.map(|a| a.messages().clone()))
+            .unwrap_or_else(|| {
+                vec![
+                    ExternalMsg::SwitchModeBuiltin("recover".into()),
+                    ExternalMsg::LogWarning(
+                        "Key map not found. Let's calm down, escape, and try again.".into(),
+                    ),
+                    ExternalMsg::Refresh,
+                ]
+            });
 
         for msg in msgs {
             self = self.enqueue(Task::new(MsgIn::External(msg), Some(key)));
@@ -1856,6 +1878,7 @@ impl App {
         } else {
             self.input_buffer = Some(input.to_owned());
         };
+        self.logs_hidden = true;
         self.refresh()
     }
 
@@ -1869,17 +1892,19 @@ impl App {
 
     fn set_input_buffer(mut self, string: String) -> Result<Self> {
         self.input_buffer = Some(string);
-        self.msg_out.push_back(MsgOut::Refresh);
-        Ok(self)
+        self.logs_hidden = true;
+        self.refresh()
     }
 
     fn remove_input_buffer_last_character(mut self) -> Result<Self> {
         if let Some(mut buf) = self.input_buffer {
             buf.pop();
             self.input_buffer = Some(buf);
-        };
-        self.msg_out.push_back(MsgOut::Refresh);
-        Ok(self)
+            self.logs_hidden = true;
+            self.refresh()
+        } else {
+            Ok(self)
+        }
     }
 
     fn remove_input_buffer_last_word(mut self) -> Result<Self> {
@@ -1896,23 +1921,25 @@ impl App {
                 .collect::<String>();
 
             self.input_buffer = Some(buf);
-        };
-        self.msg_out.push_back(MsgOut::Refresh);
-        Ok(self)
+            self.logs_hidden = true;
+            self.refresh()
+        } else {
+            Ok(self)
+        }
     }
 
     fn reset_input_buffer(mut self) -> Result<Self> {
         self.input_buffer = None;
-        self.msg_out.push_back(MsgOut::Refresh);
-        Ok(self)
+        self.refresh()
     }
 
     fn focus_by_index(mut self, index: usize) -> Result<Self> {
         if let Some(dir) = self.directory_buffer_mut() {
             dir.focus = index.min(dir.total.max(1) - 1);
-            self.msg_out.push_back(MsgOut::Refresh);
-        };
-        Ok(self)
+            self.refresh()
+        } else {
+            Ok(self)
+        }
     }
 
     fn focus_by_index_from_input(self) -> Result<Self> {
@@ -1934,12 +1961,13 @@ impl App {
                 .map(|(i, _)| i)
             {
                 dir_buf.focus = focus;
-                self.msg_out.push_back(MsgOut::Refresh);
+                self.refresh()
             } else {
-                self = self.log_error(format!("{} not found in $PWD", name))?;
+                self.log_error(format!("{} not found in $PWD", name))
             }
-        };
-        Ok(self)
+        } else {
+            Ok(self)
+        }
     }
 
     fn focus_path(self, path: &str) -> Result<Self> {
@@ -1964,13 +1992,34 @@ impl App {
         }
     }
 
+    fn push_mode(mut self) -> Self {
+        if self.mode() != self.config().modes().builtin().recover()
+            && self
+                .last_modes
+                .last()
+                .map(|m| m != self.mode())
+                .unwrap_or(true)
+        {
+            self.last_modes.push(self.mode.clone())
+        }
+        self
+    }
+
+    fn pop_mode(mut self) -> Result<Self> {
+        if let Some(mode) = self.last_modes.pop() {
+            self.input_buffer = None;
+            self.mode = mode;
+        };
+        Ok(self)
+    }
+
     fn switch_mode(mut self, mode: &str) -> Result<Self> {
         if let Some(mode) = self.config().modes().clone().get(mode) {
+            self = self.push_mode();
             self.input_buffer = None;
             self.mode = mode
                 .to_owned()
                 .sanitized(self.config().general().read_only().unwrap_or_default());
-            self.msg_out.push_back(MsgOut::Refresh);
             Ok(self)
         } else {
             self.log_error(format!("Mode not found: {}", mode))
@@ -1979,11 +2028,11 @@ impl App {
 
     fn switch_mode_builtin(mut self, mode: &str) -> Result<Self> {
         if let Some(mode) = self.config().modes().clone().get_builtin(mode) {
+            self = self.push_mode();
             self.input_buffer = None;
             self.mode = mode
                 .to_owned()
                 .sanitized(self.config().general().read_only().unwrap_or_default());
-            self.msg_out.push_back(MsgOut::Refresh);
             Ok(self)
         } else {
             self.log_error(format!("Builtin mode not found: {}", mode))
@@ -1992,11 +2041,11 @@ impl App {
 
     fn switch_mode_custom(mut self, mode: &str) -> Result<Self> {
         if let Some(mode) = self.config().modes().clone().get_custom(mode) {
+            self = self.push_mode();
             self.input_buffer = None;
             self.mode = mode
                 .to_owned()
                 .sanitized(self.config().general().read_only().unwrap_or_default());
-            self.msg_out.push_back(MsgOut::Refresh);
             Ok(self)
         } else {
             self.log_error(format!("Custom mode not found: {}", mode))
@@ -2006,8 +2055,7 @@ impl App {
     fn switch_layout(mut self, layout: &str) -> Result<Self> {
         if let Some(l) = self.config().layouts().get(layout) {
             self.layout = l.to_owned();
-            self.msg_out.push_back(MsgOut::Refresh);
-            Ok(self)
+            self.refresh()
         } else {
             self.log_error(format!("Layout not found: {}", layout))
         }
@@ -2016,8 +2064,7 @@ impl App {
     fn switch_layout_builtin(mut self, layout: &str) -> Result<Self> {
         if let Some(l) = self.config().layouts().get_builtin(layout) {
             self.layout = l.to_owned();
-            self.msg_out.push_back(MsgOut::Refresh);
-            Ok(self)
+            self.refresh()
         } else {
             self.log_error(format!("Builtin layout not found: {}", layout))
         }
@@ -2026,19 +2073,20 @@ impl App {
     fn switch_layout_custom(mut self, layout: &str) -> Result<Self> {
         if let Some(l) = self.config().layouts().get_custom(layout) {
             self.layout = l.to_owned();
-            self.msg_out.push_back(MsgOut::Refresh);
-            Ok(self)
+            self.refresh()
         } else {
             self.log_error(format!("Custom layout not found: {}", layout))
         }
     }
 
     fn call(mut self, command: Command) -> Result<Self> {
+        self.logs_hidden = true;
         self.msg_out.push_back(MsgOut::Call(command));
         Ok(self)
     }
 
     fn call_silently(mut self, command: Command) -> Result<Self> {
+        self.logs_hidden = true;
         self.msg_out.push_back(MsgOut::CallSilently(command));
         Ok(self)
     }
@@ -2271,16 +2319,25 @@ impl App {
     }
 
     pub fn log_info(mut self, message: String) -> Result<Self> {
+        self.logs_hidden = false;
         self.logs.push(Log::new(LogLevel::Info, message));
         Ok(self)
     }
 
     pub fn log_success(mut self, message: String) -> Result<Self> {
+        self.logs_hidden = false;
         self.logs.push(Log::new(LogLevel::Success, message));
         Ok(self)
     }
 
+    pub fn log_warning(mut self, message: String) -> Result<Self> {
+        self.logs_hidden = false;
+        self.logs.push(Log::new(LogLevel::Warning, message));
+        Ok(self)
+    }
+
     pub fn log_error(mut self, message: String) -> Result<Self> {
+        self.logs_hidden = false;
         self.logs.push(Log::new(LogLevel::Error, message));
         Ok(self)
     }
@@ -2433,6 +2490,7 @@ impl App {
 
         [
             builtin.default(),
+            builtin.recover(),
             builtin.filter(),
             builtin.number(),
             builtin.go_to(),
@@ -2578,4 +2636,9 @@ impl App {
     pub fn layout(&self) -> &Layout {
         &self.layout
     }
+
+    /// Get a reference to the app's logs hidden.
+    pub fn logs_hidden(&self) -> bool {
+        self.logs_hidden
+    }
 }
